from .translated_object import TranslatedObject
from .base_translator import BaseTranslator

BASE_HEADERS: dict = {
    "User-Agent": "GoogleTranslate/6.6.1.RC09.302039986 (Linux; U; Android 9; Redmi Note 8)",
}
